function TizenSystemController(){
    if(arguments.callee._singletonInstance){
	    return arguments.callee._singletonInstance;
	}
	arguments.callee._singletonInstance = this;

	var self = this; self.name = "TizenSystemController"; // MANDATORY
    this.dataDir = "documents/";
    this.rootDir = "";
    this.packageDir = "wgt-package/";
    this.appDir = "";

    this.ready_callback = false;
    this.systemIsUpdating = false;

    /* info system */
    this.firmwareVersion = null;
    this.subMicomVersion = null;
    this.modelTV = null;
    this.id_screen = "00";
    this.serialNumber = null;
    this.dataInfoSystem = null;
    this.sendInfoSystem = false;

    this.screenshotServer = null;

    this.id_content_now = null;

    this.superEco = false;
    this.eco = false;
    this.debug = false;

    this.infoPlus = null;

    this.busy = false;
    this.timersSystem = {};
    this.SocketConnector = undefined;

    this.sync_master_supported = false;

    this.api_key = "";
    this.setApiKey = function(key){
        if(typeof(key) !== "undefined"){
            self.api_key = key;
        }
    }
    
    this.display_screens = [{'bounds':{"height" : 1080,"width" : 1920,"x" : 0,"y" : 0},'touch':0,'rotation':0,'work_area':{"height" : 1080,"width" : 1920,"x" : 0,"y" : 0}}];

    this.nodejs = false;
    this.nodejs_service = false;
    this.nodejs_service_port = 8888;
    this.nodejs_timer = 0;
    this.nodejs_status = 0;
    this.nodejs_retries = 0;

    this.websocket_server = false;
    this.websocket_onOpenCallback = false;
    this.websocket_onMessageCallback = false;
    this.websocket_onTerminatedCallback = false;

    this.system_ip = "0.0.0.0";

    this.uses_texture_graphics = false;

    this.sssp_version = 0;

    this.downloadFileCallback = false;
    
    this.cpu_usage = false;
    this.storage = false;

    this.init = function(callback){
      self.SocketConnector = new SocketConnector(self);
        self.ready_callback = callback;
        self.getRootDir();
        var download_api_capability = tizen.systeminfo.getCapability("http://tizen.org/feature/download");
        if(download_api_capability === false) {
            console.log("Download API is not supported on this device.");
        }
        self.findVersionSSSP();
        self.startServiceNodeJS();
    };

    this.findVersionSSSP = function(){
      if(self.sssp_version == 0){
        var systemInfo = navigator.userAgent || navigator.vendor || window.opera;
        systemInfo = systemInfo.toLowerCase();
        self.log("SystemInfo : "+systemInfo);
        var tizen_version = systemInfo.substring(systemInfo.lastIndexOf("tizen"),systemInfo.lastIndexOf("tizen")+11);
        tizen_version = parseInt(tizen_version.substring(tizen_version.length-5,tizen_version.length));
        self.log("Tizen Version : "+tizen_version);
        if(tizen_version > 3){
          self.sssp_version = 6;
          self.sync_master_supported = true;
        }else if(tizen_version > 2){
          self.sssp_version = 5;
          self.sync_master_supported = true;
        }else{
          self.sssp_version = 4;
          self.sync_master_supported = true;
        }
      }
    };

    this.checkServiceNodeJS = function(){
      clearTimeout(self.nodejs_timer);
      self.log("[NodeJS Checker] : Status = "+self.nodejs_status);
      if(self.nodejs_status != 1){
        if(self.nodejs_retries >= 5){
          self.nodejs_retries = 0;
          self.nodejs_service = null;
          self.nodejs_service = false;
          self.startServiceNodeJS();
          self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+9,self.name+":[NodeJS] : NodeJS is not responding ... restarting");
        }else{
          self.nodejs_retries++;
          if(self.nodejs_service){
            self.nodejs_service.emit("status","get");
          }
          self.nodejs_timer = setTimeout(self.checkServiceNodeJS,1000);
        }
      }else{
        self.nodejs_service.on('onFileHash', self.onFileHash);
        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+10,self.name+":[NodeJS] : NodeJS Started Correctly");
      }
    };

    this.onStatusNodeJS = function(data){
      if(typeof(data.status) !== "undefined"){
        if(data.status == "ok"){
          self.nodejs_status = 1;
        }else if(data.status == "error"){
          self.nodejs_status = -1;
        }
      }
    };

    this.startServiceNodeJS = function(){
      if(!self.nodejs_service){
        var myIp = null;
        try {
          myIp = b2bapis.b2bcontrol.getIPAddress();
          if(self.system_ip != myIp){
            self.system_ip = myIp;
          }
        }catch(e){
          self.log("[getIPAddress] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
        }
        if(self.system_ip != "0.0.0.0"){
          if(self.sssp_version == 6){
            self.log("[NodeJS] : Trying to connect to : http://" + self.system_ip + ':' + self.nodejs_service_port)
            self.nodejs_service = io.connect('http://' + self.system_ip + ':' + self.nodejs_service_port);
            self.nodejs_service.on('onFileHash', self.onFileHash);
            self.nodejs_service.on('onLoadExternalApp', self.onLoadExternalApp);
            self.nodejs_service.on('onStatus', self.onStatusNodeJS);
            self.checkServiceNodeJS();
          }else{
            try{
              try{
                b2bapis.b2bcontrol.stopNodeServer(function(){
                  self.log("stopNodeServer success");
                  b2bapis.b2bcontrol.startNodeServer("../server.js", "SSSPServer", function(){
                    self.log("[NodeJS] : Trying to connect to : http://" + self.system_ip + ':' + self.nodejs_service_port)
                    self.nodejs_service = io.connect('http://' + self.system_ip + ':' + self.nodejs_service_port);
                    self.nodejs_service.on('onFileHash', self.onFileHash);
                    self.nodejs_service.on('onLoadExternalApp', self.onLoadExternalApp);
                    self.nodejs_service.on('onStatus', self.onStatusNodeJS);
                    self.checkServiceNodeJS();
                  },function(error){
                    self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+8,self.name+":[NodeJS] : NodeJS could not get started 1");
                    self.checkServiceNodeJS();
                  });
                },function(err){
                  self.log("stopNodeServer failed Error: "+err.message);
                  b2bapis.b2bcontrol.startNodeServer("../server.js", "SSSPServer", function(){
                    self.log("[NodeJS] : Trying to connect to : http://" + self.system_ip + ':' + self.nodejs_service_port)
                    self.nodejs_service = io.connect('http://' + self.system_ip + ':' + self.nodejs_service_port);
                    self.nodejs_service.on('onFileHash', self.onFileHash);
                    self.nodejs_service.on('onLoadExternalApp', self.onLoadExternalApp);
                    self.nodejs_service.on('onStatus', self.onStatusNodeJS);
                    self.checkServiceNodeJS();
                  },function(error){
                    self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+8,self.name+":[NodeJS] : NodeJS could not get started 2");
                    self.checkServiceNodeJS();
                  });
                });
              }catch(e){
                self.log("stopNodeServer failed Exception: ");
                b2bapis.b2bcontrol.startNodeServer("../server.js", "SSSPServer", function(){
                  self.log("[NodeJS] : Trying to connect to : http://" + self.system_ip + ':' + self.nodejs_service_port)
                  self.nodejs_service = io.connect('http://' + self.system_ip + ':' + self.nodejs_service_port);
                  self.nodejs_service.on('onFileHash', self.onFileHash);
                  self.nodejs_service.on('onLoadExternalApp', self.onLoadExternalApp);
                  self.nodejs_service.on('onStatus', self.onStatusNodeJS);
                  self.checkServiceNodeJS();
                },function(error){
                  self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+8,self.name+":[NodeJS] : NodeJS could not get started 3");
                  self.checkServiceNodeJS();
                });
              }
            }catch(e){
                self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+8,self.name+":[NodeJS] : Exception when trying to start");
                self.checkServiceNodeJS();
            }
          }
        }else{
          self.checkServiceNodeJS();
        }
      }
    };

    this.getRootDir = function(){
        tizen.filesystem.resolve(
            self.dataDir,
            function(dir) {
                self.rootDir = dir.toURI();
               self.getAppDir();
            }, function(e) {
                self.log("Unable to read root URI prefix: ");
        },"r");
    };

    this.getAppDir = function(){
        tizen.filesystem.resolve(
            self.packageDir,
            function(dir) {
                self.appDir = dir.toURI();
                self.log("-------------------------------------------------------------------------------------------"+self.appDir);
                self.checkFolders();
                self.checkCoreFile();
            }, function(e) {
                self.log("Unable to read root URI prefix: ");
        },"r");
    };

    this.createTrayIcon = function(){
        // none
    };

    this.checkFolders = function(){
         var exists = false;
        for(var route in self.ROUTES){
            if(self.ROUTES[route].isFolder && !self.ROUTES[route].exists){
                self.makeDir(self.ROUTES[route].path);
                self.ROUTES[route].exists = exists;
            }
        }
    };

    this.checkCoreFile = function(){
        self.fileExists(self.ROUTES['CORE'].path,function(exists,file_ref){
            if(exists){
                self.checkConfigFile();
            }else{
                self.createCoreFile();
            }
        });
    };

    this.createCoreFile = function(){
        var xml_data = '<?xml version="1.0" encoding="utf-8" ?><core><config><id>0</id><api_ver>3.0</api_ver><api_url>apilb.admira.com</api_url></config><server url="apilb.admira.com"/></core>';
        self.writeFile(self.ROUTES['CORE'].path,xml_data,function(){
            self.checkConfigFile();
        });
        var style_data = '<?xml version="1.0" encoding="utf-8"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg  xmlns="http://www.w3.org/2000/svg"  version="1.1" id="SVG_1"  x="0px" y="0px" enable-background="new 0 0 100% 100%" style="background-color:#000"><circle cx="98%" cy="98%" r="0.5%" style="stroke: #68983F; fill: #68983F;"><animate id="animation1" attributeName="opacity" from="0" to="1" dur="1s" begin="0s;animation2.end" /><animate id="animation2" attributeName="opacity" from="1" to="0" dur="1s" begin="animation1.end" /></circle></svg>';
        self.writeFile(self.ROUTES['STYLE'].path+"logo.svg",style_data,function(){
           self.log("LOGO File created ! ");
        });
    };

    this.checkConfigFile = function(){
        self.fileExists(self.ROUTES['CONFIG'].path,function(exists,file_ref){
            if(exists){
                self.checkDataXmlFile();
            }else{
                self.createConfigFile();
            }
        });
    };

    this.createConfigFile = function(){
        var xml_data = '<config><data file="datos.xml"/><paths multiplayer_delimiter="_" xmls="xmls/" content="content/" temp="temp/" modules="modules/" log="log/" stadistics="stats/"/><params bumper_content="assets/downloading.swf" bumper_duration="56" alwaysontop="1"/></config>';
        self.writeFile(self.ROUTES['CONFIG'].path,xml_data,function(){
            self.checkDataXmlFile();
        });
    };

    this.checkDataXmlFile = function(){
        var filename = self.getDayOfWeek()+".xml";
        self.fileExists(self.ROUTES['XML'].path+filename,function(exists,file_ref){
            if(exists){
                self.ready_callback();
                //console.log("READY");
            }else{
                self.createDataXmlFile();
            }
        });
    };

    this.createDataXmlFile = function(){
        var filename = self.getDayOfWeek()+".xml";
        var xml_data = '<?xml version="1.0"?><admira timestamp="1" version="1.0" status="ok"><archive/><playlist><loop/></playlist><screens /><config volume="100" fullscreen="true" mode="1" always_on_top="true" plv_name="Debug Name" project_name="Debug Project" time="30" plv_id="0" quit_on_click="true" activated="false" domain="" api_key="" live_url="" debug="true" live_recheck_seconds="" live_send_id=""><resolution x="100" y="100" width="300" height="300" /></config></admira>';
        self.writeFile(self.ROUTES['XML'].path+filename,xml_data,function(){
            self.ready_callback();
        });
    }

    this.writeFile = function(filename,value,callback,params){
        var subfolder = filename.substring(0,filename.lastIndexOf("/"));
        filename = filename.substring(filename.lastIndexOf("/")+1,filename.length);

        //self.log("Resolve : "+self.dataDir+subfolder);
        //self.log("Filename : "+filename);
        tizen.filesystem.resolve(
            self.dataDir+subfolder,
            function(dir) {
                var file = false;
                try{
                    file = dir.createFile(filename);
                }catch(e){
                    file = dir.resolve(filename);
                }
                self.log("writeFile : "+file.name+" in "+file.path);
                self.log("URI : "+file.toURI());
                file.openStream("w",function(fs){
                     self.log("stream open ... ");
                     try{
                        fs.write(value);
                        fs.close();
                        //console.log("written data : "+value);
                        //self.log("......................................... write!!!! "+filename);
                        if(typeof(callback) !== "undefined"){
                            callback("ok",filename,params);
                        }
                    }catch(e){
                        self.log("write error ...");
                        if(typeof(callback) !== "undefined"){
                            callback("error",filename,params);
                        }
                    }
                },function(e){
                    self.log("open stream error ...");
                    if(typeof(callback) !== "undefined"){
                        callback("error",filename,params);
                    }
                }, "UTF-8");
            }, function(e) {
            	self.log(JSON.stringify(e));
                if(typeof(callback) !== "undefined"){
                    callback("error",filename,params);
                }
        }, "rw");
    };

    this.appendFile = function(filename,value,callback,params){
        var subfolder = filename.substring(0,filename.lastIndexOf("/"));
        filename = filename.substring(filename.lastIndexOf("/")+1,filename.length);

        //self.log("Resolve : "+self.dataDir+subfolder);
        //self.log("Filename : "+filename);
        tizen.filesystem.resolve(
            self.dataDir+subfolder,
            function(dir) {
                var file = false;
                try{
                    file = dir.createFile(filename);
                }catch(e){
                    file = dir.resolve(filename);
                }
                //self.log("appendFile : "+file.name+" in "+file.path);
                //self.log("URI : "+file.toURI());
                file.openStream("a",function(fs){
                     //self.log("stream open ... ");
                     try{
                        fs.write(value);
                        fs.close();
                        //console.log("appended data : "+value);
                        if(typeof(callback) !== "undefined"){
                            callback("ok",filename,params);
                        }
                    }catch(e){
                        self.log("append error ...");
                        if(typeof(callback) !== "undefined"){
                            callback("error",filename,params);
                        }
                    }
                },function(e){
                    self.log("open stream error ...");
                    if(typeof(callback) !== "undefined"){
                        callback("error",filename,params);
                    }
                }, "UTF-8");
            }, function(e) {
                if(typeof(callback) !== "undefined"){
                    callback("error",filename,params);
                }
        }, "rw");
    };


    this.readFile = function(filename,callback,params){
      var rootDir = self.dataDir;
      if(typeof(params) !== "undefined"){
        if(typeof(params.absolute_path) !== "undefined"){
          if(params.absolute_path){
            rootDir = "";
          }
        }
      }
      tizen.filesystem.resolve(
        rootDir+filename,
        function(file) {
          file.readAsText(function(str){
            if(typeof(callback) !== "undefined"){
              callback(str,filename,params);
            }
          },function(e){
            if(typeof(callback) !== "undefined"){
              callback("error",filename,params);
            }
          }, "UTF-8");
        }, function(e) {
          self.log("read error ... ");
          if(typeof(callback) !== "undefined"){
            callback("",filename,params);
          }
        }, "r");
    };

    this.fileHash = function(filename,callback,params){
      self.findVersionSSSP();
      try{
        if(self.nodejs_service){
          var options = {"filename":filename,"params":params};
          self.nodejs_service.emit("fileHash",options);
          if(typeof(callback) !== "undefined"){
              callback(1,filename,params);
          }
        }else{
          self.startServiceNodeJS();
          if(typeof(callback) !== "undefined"){
              callback(0,filename,params);
          }
        }
      }catch(e){
        if(typeof(callback) !== "undefined"){
            callback(-1,filename,params);
        }
      }
    };

    this.onFileHash = function(data){
      self.log("[NodeJS] - onFileHash : "+JSON.stringify(data));
      var filename = "";
      var destFolder = "";
      var stored_hash = "";
      var sha1 = "";
      var fullPath = "";
      var params = undefined;
      var error = "";
      if(typeof(data.params.params) != "undefined"){
        params = data.params.params;
        if(typeof(data.params.params.hash) != "undefined"){
          stored_hash = data.params.params.hash;
        }
        if(typeof(data.params.params.destFolder) != "undefined"){
          destFolder = data.params.params.destFolder;
        }
        if(typeof(data.params.params.fullPath) != "undefined"){
          fullPath = data.params.params.fullPath;
          filename = fullPath.substring(fullPath.lastIndexOf("/")+1,fullPath.length);
        }
        if(typeof(data.msg) != "undefined"){
          sha1 = data.msg;
          error = data.msg;
        }
      }
      if(data.status == "ok"){
        /*
        self.log("[NodeJS] - onFileHash - Filename : "+filename);
        self.log("[NodeJS] - onFileHash - FullPath : "+fullPath);
        self.log("[NodeJS] - onFileHash - DestFolder : "+destFolder);
        self.log("[NodeJS] - onFileHash - XML Hash : "+stored_hash);
        self.log("[NodeJS] - onFileHash - SHA1 calc : "+sha1);
        */
        if(stored_hash.length!=40){
          self.moveVerifiedDownloadedFile(filename,fullPath,destFolder,self.downloadFileCallback,params);
          self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+13,self.name+":[DownloadFile] : File downloaded with no STORED HASH : "+filename);
        }else{
          if(stored_hash != sha1){
            self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+14,self.name+": Tizen Version : "+self.sssp_version+" -SHA1 Check failed ...  XML = "+stored_hash+" Downloaded : "+sha1+" for file : "+filename);
            if(typeof(self.downloadFileCallback) != "undefined"){
              self.downloadFileCallback(false,filename,params);
            }
          }else{
            self.moveVerifiedDownloadedFile(filename,fullPath,destFolder,self.downloadFileCallback,params);
            self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+15,self.name+":[DownloadFile] : File downloaded with HASH OK : "+filename);
          }
        }
        if(self.sssp_version == 4){
          // DELETE TEMP FILE FROM NODEJS FOLDER "/opt/share/URLLauncher/" FOR HASH CALCULATION
          tizen.filesystem.resolve(
          	"/opt/share/URLLauncher/",
              function(dir) {
                  //self.log("Trying to delete File : "+dir.fullPath+filename);
                  dir.deleteFile(dir.fullPath+filename, function(){
                      self.log("Deleted File "+dir.fullPath+filename);
                  },function(e){
                      self.log("Deleted File Error : "+dir.fullPath+filename+"    "+JSON.stringify(e));
                  });
              }, function(e) {
                  self.log("No Resolve - Trying to delete File : "+dir.fullPath+filename);
                  self.log(JSON.stringify(e))
          }, "rw");
        }
      }else{
        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+16,self.name+": Tizen Version : "+self.sssp_version+" - Could not read file hash for : "+filename+ "Error : "+error);
        if(typeof(self.downloadFileCallback) != "undefined"){
          self.downloadFileCallback(false,filename,params);
        }
      }
    };
    
    this.loadExternalApp = function(filename,callback,params){
      self.findVersionSSSP();
      if(self.nodejs_service){
        var data_dir = "";
        if(self.sssp_version == 4){
          data_dir = "/opt/share/URLLauncher/";
        }else{
          data_dir = "/opt/usr/home/owner/content/Documents/";
        }
        var options = {"filename":data_dir+filename,"params":params};
        self.nodejs_service.emit("loadExternalApp",options);
      }else{
        self.startServiceNodeJS();
      }

      if(typeof(callback) !== "undefined"){
          callback("0",filename,params);
      }
    };
    
    this.onLoadExternalApp = function(data){
      self.log("[NodeJS] - onLoadExternalApp : "+JSON.stringify(data));
    };

    this.fileExists = function(filename,callback,params){
        //self.log("check is exists : "+self.dataDir+filename);
        tizen.filesystem.resolve(
            self.dataDir+filename,
            function(file){
                //self.log("YES fileExists with URI : "+file.toURI());
                if(typeof(callback) !== "undefined"){
                    callback(true,filename,params);
                }
            },function(e){
                //self.log("Resolve Error : "+e.message);
                self.log("NO fileExists : "+self.dataDir+filename);
                if(typeof(callback) !== "undefined"){
                    callback(false,filename,params);
                }
        },"r");
    };

    this.makeDir = function(path, callback, params){
        tizen.filesystem.resolve(
            self.dataDir,
            function(dir) {
                try{
                    dir.createDirectory(path);
                    self.log("Created : "+path);
                    if(typeof(callback) !== "undefined"){
                    	callback(true,path,params);
                    }
                }catch(e){
                    self.log("Exists : "+path);
                    if(typeof(callback) !== "undefined"){
                        callback(true,path,params);
                    }
                }

            }, function(e) {
                self.log("Exists : "+path);
                 console.log("Error Creating : "+path);
                if(typeof(callback) !== "undefined"){

                    callback(false,path,params);
                }
        },"rw");
    };

    this.modificationDate = function(filename,callback,params){
        tizen.filesystem.resolve(
            self.dataDir+filename,
            function(file) {
                var date = new Date(file.modified).getTime();
                //self.log("modificationDate : "+date);
                if(typeof(callback) !== "undefined"){
                    callback(date,filename,params);
                }
            }, function(e) {
                if(typeof(callback) !== "undefined"){
                    callback(1,filename,params);
                }
        },"r");
    };

    this.downloadFile = function(url,destFolder,callback,params){
        var filename = url;
        self.log("start download : "+filename+" from : "+url);
        filename = filename.substring(filename.lastIndexOf("/")+1,filename.length);
        var stored_hash = "";
        if(typeof(params) !== "undefined"){
            if(typeof(params.filename) !== "undefined"){
                filename = params.filename;
            }
            if(typeof(params.hash) !== "undefined"){
                stored_hash = params.hash;
            }
        }
        try{
            var requestListener = {
                onprogress: function(id, receivedSize, totalSize){
                    var percent = 0;
                    if(totalSize > 0 && receivedSize > 0){
                      percent = Math.round(100*(receivedSize/totalSize));
                    }
                    self.log('Received with id: ' + id + ', ' + receivedSize + '/' + totalSize + ' - ' + percent + '%');
                },
                onpaused: function(id) {
                    self.log('Paused with id: ' + id);
                },
                oncanceled: function(id) {
                    self.log('Canceled with id: ' + id);
                    if(typeof(callback) !== "undefined"){
                        callback(false,filename,params);
                    }
                },
                oncompleted: function(id, fullPath) {
                      self.log('Completed with id: ' + id + ', full path: ' + fullPath);
                      if(self.nodejs_status != 1 || stored_hash.length != 40){
                        self.log("No NODEJS Active or NO STORED HASH");
                        self.log("NodeJS Status : "+self.nodejs_status+" Stored Hash : "+stored_hash);
                        self.moveVerifiedDownloadedFile(filename,fullPath,destFolder,callback,params);
                        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+11,self.name+":[DownloadFile] : File downloaded with NodeJS not activated or no HASH ["+stored_hash+"]");
                      }else{
                        self.verifyDownloadedFile(filename,fullPath,destFolder,function(can_be_verified){
                          self.log("Can verify file : "+can_be_verified);
                          if(!can_be_verified){
                            self.moveVerifiedDownloadedFile(filename,fullPath,destFolder,callback,params);
                            self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+12,self.name+":[DownloadFile] : File downloaded with no HASH verification");
                          }else{
                            self.log("Verifying file .... waitting for onFileHash Event");
                            self.downloadFileCallback = callback;
                          }
                        },params);
                      }
                },
                onfailed: function(id, error) {
                    self.log('Failed with id: ' + id + ', error name: ' + error.name);
                    self.log(JSON.stringify(error));
                    if(typeof(callback) !== "undefined"){
                        callback(false,filename,params);
                    }
                }
            };

            tizen.filesystem.resolve(self.dataDir,function(dir) {
                    dir.deleteFile(self.dataDir+self.ROUTES['TMP'].path+filename);
                    var downloadRequest = new tizen.DownloadRequest(url, self.dataDir+self.ROUTES['TMP'].path, filename);
                    downloadRequest.httpHeader["apikey"] = "API-"+self.core_id+"-"+self.api_key;
                    self.log("...............................2 .comenzamos la descarga");
                    tizen.download.start(downloadRequest, requestListener);
                }, function(e) {
                	self.log("Resolve Failed before download ... ");
                    if(typeof(callback) !== "undefined"){
                        callback(false,filename,params);
                    }
            },"rw");
        }
        catch(e){
            self.log(".......................... error download");
            self.log(JSON.stringify(e));
        }
    };

    this.moveVerifiedDownloadedFile = function(filename,fullPath,destFolder,callback,params){
      var timestamp = 1;
      var content_type = "";
      var content_id = 0;
      var file_ref = "";
      var stored_hash = "";
      if(typeof(params) !== "undefined"){
          if(typeof(params.xml_timestamp) !== "undefined"){
              timestamp = params.xml_timestamp;
          }
          if(typeof(params.content_id) !== "undefined"){
              content_id = params.content_id;
          }
          if(typeof(params.content_type) !== "undefined"){
              content_type = params.content_type;
          }
          if(typeof(params.file_ref) !== "undefined"){
              file_ref = params.file_ref;
          }
          if(typeof(params.hash) !== "undefined"){
              stored_hash = params.hash;
          }
      }
      tizen.filesystem.resolve(
          self.dataDir,
          function(dir) {
                self.log('Moving file :'+fullPath+" to : "+self.dataDir+destFolder+filename);
                dir.moveTo(fullPath,self.dataDir+destFolder+filename,true,function() {
                    self.log("file moved");
                    var hash = "";
                    var params_result = {};
                    var ok = true;
                    if(ok){
                        self.log("downloadFile OK");
                        params_result = {'filename':filename,
                                      'xml_timestamp':timestamp,
                                      'content_id':content_id,
                                      'content_type':content_type,
                                      'file_ref':file_ref,
                                      'sha1':hash
                                     };
                    }
                    if(typeof(callback) !== "undefined"){
                        callback(true,filename,params_result);
                    }
                },function(e){
                    self.log("file moving error : "+e.message);
                    if(typeof(callback) !== "undefined"){
                        callback(false,filename,params);
                    }
                });
          }, function(e) {
              self.log("Moving file failed !");
              self.log(JSON.stringify(e));
              if(typeof(callback) !== "undefined"){
                  callback(false,filename,params);
              }
      },"rw");
    };

    this.verifyDownloadedFile = function(filename,fullPath,destFolder,callback,params){
      try{
        tizen.filesystem.resolve(self.dataDir,function(dir){
              // EMITS FILE HASH CHECK TO NODEJS ... please see onFileHash Event.
              if(typeof(params) != "undefined"){
                params.destFolder = destFolder;
                params.fullPath = fullPath;
              }
              if(self.sssp_version == 4){
                  dir.copyTo(fullPath,"/opt/share/URLLauncher/"+filename,true,function(){
                    self.log("File copied to Hash Folder Check");
                    self.fileHash("/opt/share/URLLauncher/"+filename,function(status){
                      self.log("HASH Test status: "+status);
                      if(status == 1){
                        if(typeof(callback) !== "undefined"){
                            callback(true,filename,params);
                        }
                      }else{
                        if(typeof(callback) !== "undefined"){
                            callback(false,filename,params);
                        }
                      }
                    },params);
                  },function(){
                    self.log("File NOT moved to Hash Folder Check");
                    if(typeof(callback) !== "undefined"){
                        callback(false,filename,params);
                    }
                  });
              }else if(self.sssp_version == 5){
                self.log("SSSP5 does not support file hash calculation");
                if(typeof(callback) !== "undefined"){
                    callback(false,filename,params);
                }
              }else{
                self.fileHash(fullPath,function(status){
                  if(status == 1){
                    self.log("HASH Test status: "+status);
                    if(typeof(callback) !== "undefined"){
                        callback(true,filename,params);
                    }
                  }else{
                    if(typeof(callback) !== "undefined"){
                        callback(false,filename,params);
                    }
                  }
                },params);
              }
        },function(e) {
                self.log("Moving file failed !");
                self.log(JSON.stringify(e));
                if(typeof(callback) !== "undefined"){
                    callback(false,filename,params);
                }
        },"rw");
      }catch(e){
        if(typeof(callback) !== "undefined"){
            callback(false,filename,params);
        }
      }
    };

    this.unzipFile = function(filename,destFolder,callback,params){
  		self.log("Opening Zip File: "+filename);
      self.makeDir(destFolder,function(ok){
        if(ok){
          tizen.archive.open(self.dataDir+filename, "rw", function(archive){
              self.log("Extracting Zip to : "+self.dataDir+destFolder);
              operationId = archive.extractAll(self.dataDir+destFolder, function(){
                  if(typeof(callback) !== "undefined"){
                      callback(true,filename,params);
                  }
              },function(e){
              	self.log(JSON.stringify(e));
                  if(typeof(callback) !== "undefined"){
                      callback(false,filename,params);
                  }
              },function(opId, val, name){
                  //progress
                 // self.log("extracting operation (: " + opId + ") is in progress (" + (val * 100).toFixed(1) + "%)");
              },true);
              //tizen.archive.abort(operationId);
              //self.log("Extracting completed ");
          });
        }else{
          if(typeof(callback) !== "undefined"){
              callback(false,filename,params);
          }
        }
      });
    };

    this.listFilesDirectory=function(directory, callback){
      tizen.filesystem.resolve(
          directory,
          function(dir) {
              dir.listFiles(function(files){
                  self.log(JSON.stringify(files));

                  for(var i=0;i<files.length;i++){
                      self.log("File on "+dir.fullPath+" : "+files[i].name);
                  }

                  if(typeof(callback) !== "undefined"){
                      callback();
                  }
              }, function(e){
                  if(typeof(callback) !== "undefined"){
                      callback();
                  }
              });
          }, function(e) {
              if(typeof(callback) !== "undefined"){
                  callback();
              }
      }, "r");
    }
    this.readDirectory = function(directory,callback,params){
        tizen.filesystem.resolve(
            self.dataDir+directory,
            function(dir) {
                dir.listFiles(function(files){
                    var fileList = [];

                    for(var i=0;i<files.length;i++){
                        console.log("File on "+dir.fullPath+" : "+files[i].name);
                        fileList.push(files[i].name);
                    }

                    if(typeof(callback) !== "undefined"){
                        callback(fileList,directory,params);
                    }
                }, function(e){
                    if(typeof(callback) !== "undefined"){
                        callback([],directory,params);
                    }
                });
            }, function(e) {
                if(typeof(callback) !== "undefined"){
                    callback([],directory,params);
                }
        }, "r");
    };

    this.watchDirectory = function(directory,compare_files,callback,params){
        // TODO
    };

    this.deleteFile = function(filename,callback,params){
        var subfolder = filename.substring(0,filename.lastIndexOf("/"));
        //self.log("documents/"+filename);
        //filename = filename.substring(filename.lastIndexOf("/")+1,filename.length);
        tizen.filesystem.resolve(
        	self.dataDir,
            function(dir) {
                //self.log("Trying to delete File : "+dir.fullPath+filename);
                dir.deleteFile(dir.fullPath+filename, function(){
                    self.log("Deleted File "+dir.fullPath+filename);
                    if(typeof(callback) !== "undefined"){
                        callback(true,filename,params);
                    }
                },function(e){
                    self.log("Deleted File Error : "+dir.fullPath+filename+"    "+JSON.stringify(e));
                    if(typeof(callback) !== "undefined"){
                        callback(false,filename,params);
                    }
                });
            }, function(e) {
                self.log("No Resolve - Trying to delete File : "+dir.fullPath+filename);
                self.log(JSON.stringify(e))
                if(typeof(callback) !== "undefined"){
                    callback(false,filename,params);
                }
        }, "rw");
    };

    this.deleteFilesInDir = function(dir,filter,callback,params){
      var documentsDir;
  	  function onsuccess(files) {
  		  if(files.length > 0){
  			for (var i = 0; i < files.length; i++) {
  	          self.log(files[i].fullPath);
  	          if (files[i].isDirectory) {
  	            documentsDir.deleteDirectory(
  	              files[i].fullPath,
  	              true,
  	              function() {
  	            	  self.log("Directory Deleted");
  	            	  if(i >= files.length-1){
  	            		  self.log("Directory Deleted................................fin");
  	            		  if(typeof(callback) !== "undefined"){
  	            			  callback(null, dir, files.length);
  	            		  }
  	            	  }
  	              }, function(e) {
  	                self.log("Error");
  	                self.log(JSON.stringify(e));
  								/*	if(typeof(callback) !== "undefined"){
  	                  callback(e, dir, files.length);
  						      }*/
  	              }
  	            );
  			      }
  	          else{
  	            documentsDir.deleteFile(files[i].fullPath, function(){
  	              self.log("Deleted File ");
  	              if(i >= files.length-1){
  	                if(typeof(callback) !== "undefined"){
  	                  callback(null,dir,files.length);
  	                }
  	              }
  	            },function(e){
  	              self.log("Deleted File Error : "+JSON.stringify(e));
  	              if(i >= files.length-1){
  	                if(typeof(callback) !== "undefined"){
  	                  callback(null,dir,files.length);
  	                }
  	              }
  	            });
  	          }
  	        }
  		  }
  		  else{
  			if(typeof(callback) !== "undefined"){
              callback(null,dir,files.length);
            }
  		  }

      }
      function onerror(error) {
        self.log("The error " + error.message + " occurred when listing the files in the selected folder");
        if(typeof(callback) !== "undefined"){
          callback(error, dir, params);
        }
      }
      tizen.filesystem.resolve(
        dir,
        function(dir1) {
          documentsDir = dir1;
          dir1.listFiles(onsuccess,onerror);
        }, function(e) {
          self.log("...."+JSON.stringify(e));
          if(typeof(callback) !== "undefined"){
            callback(e, dir, 0);
          }
        }, "rw");
    };

    this.moveFile = function(source,destination,callback,params){
      tizen.filesystem.resolve(
        self.dataDir,
          function(dir) {
            self.log('Copy file :'+source+" to : "+destination);
              dir.moveTo(self.dataDir+source,self.dataDir+destination, true, function(){
                self.log("file move");
                  if(typeof(callback) !== "undefined"){
                    callback(true,source,params);
                  }
              }, function(e){
                self.log("file copy error : "+e.message);
                  if(typeof(callback) !== "undefined"){
                    callback(false,source,params);
                  }
              });
          }, function(e) {
            self.log("Move file failed !");
            self.log(e)
              if(typeof(callback) !== "undefined"){
                callback(false,source,params);
              }
      },"rw");
    };

    this.moveTizenFile = function(source,destination,callback,params){
        //TODO
      tizen.filesystem.resolve(
        self.dataDir,
          function(dir) {
            self.log('Copy file :'+source+" to : "+destination);
              dir.moveTo(source,destination, true, function(){
                self.log("file move");
                  if(typeof(callback) !== "undefined"){
                    callback(false,true,params);
                  }
              }, function(e){
                self.log("file copy error : "+e.message);
                  if(typeof(callback) !== "undefined"){
                    callback(true,"",params);
                  }
              });
          }, function(e) {
            self.log("Move file failed !");
            self.log(e)
              if(typeof(callback) !== "undefined"){
                callback(true,"",params);
              }
      },"rw");
    };
    this.systemUpdate = function(update_folder,platform){
      //update_folder = update_folder.substring(0, 23);
      //update_folder = update_folder.replace("temp/", "");
      //update_folder = "upgrade/";
      self.log("Updating the system .... ");
      var result = 0;
      var errors = false;
      self.log("Check backup : app_backup/");
      var num = 0;
      self.deleteFilesInDir(self.dataDir+"app_backup/", "", function(err, result, number){
        num++;
        if(num >= number){
          self.log("Delete Backup");
          self.moveTizenFile(self.dataDir+"app/", self.dataDir+"app_backup/", function(errM, reusltM){
            if(!errM){
              self.log("Backup Created");
              self.moveTizenFile(self.dataDir+update_folder+"files/app/",self.dataDir+"app/", function(err3, result3){
                if(!err3){
                  self.log(result+" : Moving "+update_folder+"files/app"+" to app/");
                  self.log("System Update complete ! rebooting !");
                  self.readFile(update_folder+"files/admira.ver", function(data, filename){
                    if(data){
                      self.log(data);
                      self.writeFile("admira.ver", data, function(){
                        self.log("admira.ver write OK");
                        self.deleteDir(self.dataDir+update_folder);
                      });
                    }
                  })
                }
                else{
                  self.log("Errors Updating Software ... rollback");
                  self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+7,self.name+": Function systemUpdate() failed : Errors Updating Software ... rollback");
                  self.moveTizenFile("app_backup/"+self.ROUTES['MAIN_HTML'].path, self.ROUTES['MAIN_HTML'].path, function(err5, result5){
                    self.systemIsUpdating = false;
                  })
                }
              })
            }
            else{
              self.log("Can not create backup ... avoiding ... ");
              self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+6,self.name+": Function systemUpdate() failed : Can not create backup ... avoiding ... ");
              self.systemIsUpdating = false;
            }
          });
        }
      });
    };

    this.systemRepairUpdate = function(){
      // TODO
    };

    this.setSystemDateTime = function(timestamp,callback,params){
      self.log("Setting Tizen Time : " + timestamp);
      var date = new Date(timestamp*1000); // In milliseconds

      var year = parseInt(moment.utc(date).format('YYYY'));
      var month = parseInt(moment.utc(date).format('M'));
      var day = parseInt(moment.utc(date).format('D'));
      var hour = parseInt(moment.utc(date).format('H'));
      var minute = parseInt(moment.utc(date).format('m'));
      var sec = parseInt(moment.utc(date).format('s'));
      var options = year + ":" + month + ":" + day + ":" + hour + ":" + minute + ":" + sec;
      self.log(options);
      function successCb() {
        if(typeof(callback) !== "undefined"){
            callback(true,params);
        }
      }
      function failureCb(cbObject) {
          self.log("Error Code [" + errorCode + "]: " + errorText);
          if(typeof(callback) !== "undefined"){
              callback(false,params);
          }
      }
      b2bapis.b2bcontrol.setCurrentTime(options, successCb, failureCb);
    };

    this.copyFile = function(source, destination, callback){
    	//self.log(".-----copyFile 1 ");
    	tizen.filesystem.resolve(
	        "/opt/",
	        function(dir) {
	        	//self.log('Copy file :'+source+" to : "+destination);
	            dir.copyTo(source,destination, true, function(){
	            	self.log("file copy");
	                if(typeof(callback) !== "undefined"){
	                    callback(true);
	                }
	            }, function(e){
	            	self.log("file copy error : "+e.message);
	                if(typeof(callback) !== "undefined"){
	                    callback(false);
	                }
	            });
	        }, function(e) {
	        	self.log("Copy file failed !");
	        	self.log(e)
	            if(typeof(callback) !== "undefined"){
	                callback(false);
	            }
	    },"r");
    }

    this.takeScreenshot = function(callback,params){
        b2bapis.b2bcontrol.captureScreen(function(data){
        	var img = new Image();
            img.crossOrigin = 'Anonymous';
            img.onload = function(){
                var base64 = false;
                var canvas = document.createElement('CANVAS');
                var context = canvas.getContext('2d');
                //250x140 Screenshot size
                var width = 250;
                var height = 140;
                if (img.height > img.width){
                    width = 140;
                    height = 250;
                }
                canvas.height = height;
                canvas.width = width;
                context.drawImage(img, 0, 0,width,height);
                if(params){
                    base64 = canvas.toDataURL("image/jpeg", params);
                }
                else{
                    base64 = canvas.toDataURL("image/jpeg", 1.0);
                }
                if(typeof(callback) !== "undefined"){
                    self.screenshotServer = base64.substring(23);
                    callback(base64.substring(22),params);
                }
                canvas = null;
            };
            var d = new Date();
            img.src = data+"?time="+d;
        }, function(err){
            console.log("Screenshot Error: "+err.message);
            if(typeof(callback) !== "undefined"){
                self.screenshotServer = null;
                callback(false,params);
            }
        });
    };
    this.getScreenshot = function(callback){
        b2bapis.b2bcontrol.captureScreen(function(data){
        	var img = new Image();
            img.crossOrigin = 'Anonymous';
            img.onload = function(){
                var base64 = false;
                var canvas = document.createElement('CANVAS');
                var context = canvas.getContext('2d');
                //250x140 Screenshot size
                var width = 250;
                var height = 140;
                if (img.height > img.width){
                    width = 140;
                    height = 250;
                }
                canvas.height = height;
                canvas.width = width;
                context.drawImage(img, 0, 0,width,height);
                base64 = canvas.toDataURL("image/jpeg", 1.0);
                if(typeof(callback) !== "undefined"){
                    self.screenshotServer = base64.substring(23);
                    callback(base64.substring(22));
                }
                canvas = null;
            };
            var d = new Date();
            img.src = data+"?time="+d;
        }, function(err){
            console.log("Screenshot Error: "+err.message);
            if(typeof(callback) !== "undefined"){
                self.screenshotServer = null;
                callback(false);
            }
        });
    };
    this.rebootSystem = function(){
        // OVERRIDE
    	self.log("reboot");
      try{
        b2bapis.b2bcontrol.stopNodeServer(function(){
          self.log("stopNodeServer success");
        },function(err){
          self.log("stopNodeServer failed Error: "+err.message);
        });
      }catch(e){
        self.log("stopNodeServer failed Exception: ");
      }
      b2bapis.b2bcontrol.rebootDevice(function(val){}, function(err){
          self.log("reboot failed Error: "+err.message);
      });
    };

    this.haltSystem = function(){
         // TODO
        b2bapis.b2bcontrol.setPowerOff(function(val){
            console.log("setPowerOff !! : ");
        }, function(err){
            console.log("setPowerOff failed Error: "+err.message);
        });
    };

    this.internetConnected = function() {
        // OVERRIDE
       // self.log("CHECK OS INTERNET CONNECTION = "+internet);

        var internet = false;
        var IPAddress = null;
        try {
    	  IPAddress = b2bapis.b2bcontrol.getIPAddress();
    	 } catch (e) {
    	  //self.log("[getIPAddress] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
    	 }
    	 if(null !== IPAddress){
    	  self.log("[getIPAddress] call syncFunction type: " + IPAddress);
    	  internet = true;
    	 }
    	 //self.log("internet: "+internet);
    	 return internet;
    };

    this.getNetworkInfo = function(callback,params){
        // TODO Improve
        var network_interfaces = [];
        try{
          network_interfaces.push({ip:b2bapis.b2bcontrol.getIPAddress(),netmask:b2bapis.b2bcontrol.getSubNetMask(),getway:b2bapis.b2bcontrol.getGatewayAddress()});
        }catch(e){
          self.log("Exception on getIPAddress");
        }
        if(typeof(callback) !== "undefined"){
          callback(network_interfaces,params);
        }
        return network_interfaces;
    };

    this.setWindow = function(x,y,w,h){
        // OVERRIDE
    };

    this.setProxyConfig = function(config){
        // OVERRIDE
    };

    this.getProxyForURL = function(url){
        // OVERRIDE
        return "";
    };

    this.getAddWhiteList = function(sourceOrigin, destinationProtocol, destinationHost, allowDestinationSubdomains){
        // OVERRIDE
    };

    this.getRemoveWhiteList = function(sourceOrigin, destinationProtocol, destinationHost, allowDestinationSubdomains){
        // OVERRIDE
    };

    this.getDataPath = function(){
         // OVERRIDE
    };

    this.setCrashDumpPath = function(folderPath){
         // OVERRIDE
    };

    this.quit = function(){
        // OVERRIDE
    };

    this.reload = function(){
      try{
        location.reload(true);
      }catch(e){
        console.log(e);
      }
    };

    this.clearCache = function(){
        // OVERRIDE
    };

    this.alwaysOnTop = function(value){
        // OVERRIDE
        self.uses_texture_graphics = !value;
    };

    this.focusWindow = function(){
        // OVERRIDE
    };

    this.disableRemoteControl = function(platform){
      self.deactivateControl();
    };

    this.enableRemoteControl = function(platform,tunnels){
      self.activateControl();
    };

    this.keepAwake = function(platform){
        // TODO : Keeps Screen Awake
    };
    
    this.deleteDirectory = function(dirname,callback,params){
      tizen.filesystem.resolve(
        self.dataDir,
          function(dir) {
              dir.deleteDirectory(dir.fullPath+dirname,true,function(){
                  self.log("Deleted Directory "+dir.fullPath+dirname);
                  if(typeof(callback) !== "undefined"){
                      callback(true,dirname,params);
                  }
              },function(e){
                  self.log("Deleted Directory Error : "+dir.fullPath+dirname+"    "+JSON.stringify(e));
                  if(typeof(callback) !== "undefined"){
                      callback(false,dirname,params);
                  }
              });
          }, function(e) {
              self.log("No Resolve - Trying to delete File : "+dir.fullPath+dirname);
              self.log(JSON.stringify(e))
              if(typeof(callback) !== "undefined"){
                  callback(false,dirname,params);
              }
      }, "rw");
    };
    

    this.deleteDir = function(dir){
    	var documentsDir;
    	  function onsuccess(files) {
    	      for (var i = 0; i < files.length; i++) {
    	    	  self.log(files[i]);
    	          if (files[i].isDirectory) {
    	              documentsDir.deleteDirectory(
    	                  files[i].fullPath,
    	                  true,
    	                  function() {
    	                	  self.log("Directory Deleted");
    	                      console.log("Directory Deleted");
    	                      if(i >= files.length-1){
                              setTimeout(function(){self.rebootSystem();},2000);
    	                      }
    	                  }, function(e) {
    	                	  self.log("Error" + e.message);
    	                     console.log("Error" + e.message);
    	              });
    	          }
    	      }
    	  }

    	  function onerror(error) {
    		  self.log("The error " + error.message + " occurred when listing the files in the selected folder");
    	  }
        self.log("delete upgrade....")
    	  tizen.filesystem.resolve(
    	      dir,
    	      function(dir) {
    	          documentsDir = dir;
    	          dir.listFiles(onsuccess,onerror);
    	      }, function(e) {
    	          self.log("Error" + e.message);
    	      }, "rw"
    	  );
    }
    this.deleteFile2 = function(){
    	function onsuccess(files) {
    		self.log(JSON.strinify(files))
    	      for (var i = 0; i < files.length; i++) {
    	          if (!files[i].isDirectory) {
    	              documentsDir.deleteFile(
    	                  files[i].fullPath,
    	                  function() {
    	                     console.log("File Deleted");
    	                  }, function(e) {
    	                     console.log("Error" + e.message);
    	              });
    	          }
    	      }
    	  }

    	  function onerror(error) {
    	      console.log("The error " + error.message + " occurred when listing the files in the selected folder");
    	  }

    	  var documentsDir;
    	  tizen.filesystem.resolve(
    			  "file:///opt/usr/media/Documents/admira.lic",
    	      function(dir) {
    	          documentsDir = dir;
    	          dir.listFiles(onsuccess,onerror);
    	      }, function(e) {
    	          console.log("Error" + e.message);
    	      }, "rw"
    	  );
    }
    this.activateControl = function(){
    	self.debug = true;
    	self.SocketConnector.createSocket();
    };

    this.activateSocketBasic = function(){
      self.SocketConnector.createSocket();
      setTimeout(function(){
        if(self.debug == false){
          self.SocketConnector.disconnectSocket();
        }
      },600000);
    };
    
    this.deactivateControl = function(){
    	self.debug = false;
    	self.SocketConnector.disconnectSocket();
    };
    this.sendControlLog = function(ori, name, text){
      if(typeof(self.SocketConnector) !== "undefined"){
          self.SocketConnector.logsSocket(ori, self.getDate(self.server_time_activated).toTimeString().split(' ')[0] +" .. " + name+ "--"+text);
      }
    };
    this.changeCore = function(id){
      self.SocketConnector.changeCore(id);
    };
    this.saveIdContent = function(idContent){
        self.id_content_now = idContent;
    };
    this.get_tunnels_params = function(){
        //self.api_connector.get(self.api_url,self.api_ver,"player/get_tunnels_params/id",self.core_id,self.remoteResponse);
    };
    this.getInfoSystem = function(callback){
    	self.firmwareVersion = webapis.productinfo.getFirmware();
    	self.subMicomVersion = webapis.productinfo.getSmartTVServerVersion();
    	self.modelTV = webapis.productinfo.getModel();
    	try {
    		self.serialNumber = b2bapis.b2bcontrol.getSerialNumber();
    	} catch (e) {
    		self.log("[getSerialNumber] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
    	}
    	/*self.dataInfoSystem = {
            newDate: new Date(),
            getDate : self.getDate(),
            getDateTrue : self.getDate(true),
            firmwareVersion: self.firmwareVersion,
            subMicomVersion : self.subMicomVersion,
            modelTV : self.modelTV,
            serialNumber : self.serialNumber
        };*/
        self.dataInfoSystem = {
            newDate: new Date(),
            getDate : self.getDate(self.server_time_activated),
            firmwareVersion: self.firmwareVersion,
            subMicomVersion : self.subMicomVersion,
            modelTV : self.modelTV,
            serialNumber : self.serialNumber
        };
        if(callback){
            callback(JSON.stringify(self.dataInfoSystem));
        }
    };

    this.getInfoSystemPlus = function(callback){
    	self.infoPlus = {
                "ProductInfoSiServerType" : JSON.stringify(webapis.productinfo.ProductInfoSiServerType),
                "Firmware" : webapis.productinfo.getFirmware(),
                "LocalSet" : webapis.productinfo.getLocalSet(),
                "Psid" : webapis.productinfo.getPsid(),
                "RealModel" : webapis.productinfo.getRealModel(),
                "SmartTVServerType" : webapis.productinfo.getSmartTVServerType(),
                "SmartTVServerVersion" : webapis.productinfo.getSmartTVServerVersion(),
                "Version" : webapis.productinfo.getVersion(),
                "isSoccerModeEnabled" : webapis.productinfo.isSoccerModeEnabled(),
                "isTtvSupported" : webapis.productinfo.isTtvSupported(),
                "isUdPanelSupported" : webapis.productinfo.isUdPanelSupported()
    	}
    	self.infoPlus.URLLauncherAddress = null;
    	try {
    		self.infoPlus.URLLauncherAddress = b2bapis.b2bcontrol.getURLLauncherAddress();
    	} catch (e) {
    		self.log("[getURLLauncherAddress] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
    		self.infoPlus.URLLauncherAddress = null;
    	}
    	self.infoPlus.menuOrientation = null;
    	try {
    		self.infoPlus.menuOrientation = b2bapis.b2bcontrol.getMenuOrientation();
    	} catch (e) {
    		self.log("[getMenuOrientation] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
    		self.infoPlus.menuOrientation = null;
    	}
    	self.infoPlus.systemUpTime = null;
    	try {
    		self.infoPlus.systemUpTime = b2bapis.b2bcontrol.getSystemUpTime();
    	} catch (e) {
    		self.log("[getSystemUpTime] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
    		self.infoPlus.systemUpTime = null;
    	}

    	self.typeNetwork(function(){
        self.cpuUsage(function(){
          self.storageUsage(function(){
            self.displayInfo(function(){
              self.getDisplayOrientation(function(){
                if(typeof(callback) !== "undefined"){
                  callback();
                }
              });
            });
          });
        });
      });
    };
    
    this.typeNetwork = function(callback){
    	function onSuccessCallback(network) {
    		self.log("SystemController"," network type: " + network.networkType);
    		self.infoPlus.networkType = network.networkType;
    		self.log(JSON.stringify(self.infoPlus));
        callback(JSON.stringify(self.infoPlus));
    	}
    	function onErrorCallback(error) {
    		self.log("SystemController","Not supported: " + error.message);
    		self.infoPlus.networkType = null;
    		self.log(JSON.stringify(self.infoPlus));
        callback(JSON.stringify(self.infoPlus))
    	}
    	tizen.systeminfo.getPropertyValue("NETWORK", onSuccessCallback, onErrorCallback);
    }
    
    this.cpuUsage = function(callback){
      function onSuccessCallback(cpu) {
        self.log("CPU Usage : " + cpu.load);
        self.cpu_usage = cpu.load;
        self.infoPlus.cpu_usage = cpu.load;
        self.log(JSON.stringify(self.infoPlus));
        callback(JSON.stringify(self.infoPlus));
      }
      function onErrorCallback(error) {
        self.log("CPU Usage Not supported: " + error.message);
        self.infoPlus.cpu_usage = false;
        self.log(JSON.stringify(self.infoPlus));
        callback(JSON.stringify(self.infoPlus));
      }
      tizen.systeminfo.getPropertyValue("CPU", onSuccessCallback, onErrorCallback);
    }

    this.storageUsage = function(callback){
      function onSuccessCallback(storage) {
        self.log("Storage Usage ");
        self.storage = storage;
        self.infoPlus.storage = storage;
        self.log(JSON.stringify(self.infoPlus));
        callback(JSON.stringify(self.infoPlus));
      }
      function onErrorCallback(error) {
        self.log("Storage Usage Not supported: " + error.message);
        self.infoPlus.storage = null;
        self.log(JSON.stringify(self.infoPlus));
        callback(JSON.stringify(self.infoPlus));
      }
      tizen.systeminfo.getPropertyValue("STORAGE", onSuccessCallback, onErrorCallback);
    }
    
    this.displayInfo = function(callback){
      function onSuccessCallback(display_info) {
        self.log("Display Info");
        try{
          self.display_screens = [{
            'bounds':{"height" : display_info.physicalHeight,"width" : display_info.physicalWidth,"x" : 0,"y" : 0},
            'touch':0,
            'rotation':0,
            'work_area':{"height" : display_info.resolutionHeight,"width" : display_info.resolutionWidth,"x" : 0,"y" : 0}
          }];
        }catch(e){
          self.log("Could not set Display info obejct");
        }
        self.infoPlus.display_info = self.display_screens;
        self.log(JSON.stringify(self.infoPlus));
        callback(JSON.stringify(self.infoPlus));
      }
      function onErrorCallback(error) {
        self.log("Display Info Not supported: " + error.message);
        self.infoPlus.display_info = null;
        self.log(JSON.stringify(self.infoPlus));
        callback(JSON.stringify(self.infoPlus));
      }
      tizen.systeminfo.getPropertyValue("DISPLAY", onSuccessCallback, onErrorCallback);
    }
    
    
    
    this.setContentRotation = function(options, callback_setContentRotation) {
      
    }

    this.getDisplayOrientation = function(callback,params) {
      function onSuccessCallback(display_info) {
        self.log("Display Orientation");
        try{
          if(display_info.status == 'PORTRAIT_PRIMARY'){
            self.display_screens[0].rotation = 0;
          }else if(display_info.status == 'PORTRAIT_SECONDARY'){
            self.display_screens[0].rotation = 180;
          }else if(display_info.status == 'LANDSCAPE_PRIMARY'){
            self.display_screens[0].rotation = 270;
          }else if(display_info.status == 'LANDSCAPE_SECONDARY'){
            self.display_screens[0].rotation = 90;
          }
          self.infoPlus.display_info = self.display_screens;
        }catch(e){
          self.log("Could not set Display Orientation info obejct");
        }
        self.log(JSON.stringify(self.infoPlus));
        callback(JSON.stringify(self.infoPlus));
      }
      function onErrorCallback(error) {
        self.log("Display Orientation Not supported: " + error.message);
        self.log(JSON.stringify(self.infoPlus));
        callback(JSON.stringify(self.infoPlus));
      }
      tizen.systeminfo.getPropertyValue("DEVICE_ORIENTATION", onSuccessCallback, onErrorCallback);
    }




    this.callback_updateURL = null;
    this.updateURL = function(callback){
      self.callback_updateURL = callback;
      var origin = b2bapis.b2bcontrol.getURLLauncherAddress() + "/ADmira.wgt";
      self.downloadFile(origin,"",self.onURLDownloadComplete,callback);
    }

    this.onURLDownloadComplete = function(ok,url,callback){
        if(ok){
        	$("#starter").html("URL downloaded.");
            self.log("URL downloaded.");
            self.OS.unzipFile("ADmira.wgt", "./", function(ok,file_ref,params){
                self.OS.deleteFile("ADmira.wgt");
                if(ok){
                  self.readFile("admira.ver", function(data, filename){
                    if(data){
                    	$("#starter").html(data);
                    	self.log(data)
                      self.writeFile("url.ver", data, function(){
                        setTimeout(function(){self.rebootSystem();}, 2000);
                        self.callback_updateURL(null, "App updated from url correctly -- Reboot");
                        $("#starter").html("App updated from url correctly -- Reboot");
                      });
                    }
                    else{
                    	$("#starter").html("Updated app from url with errors -- Reboot");
                    	setTimeout(function(){self.rebootSystem();}, 2000);
                      self.callback_updateURL(null, "Updated app from url with errors -- Reboot");
                    }
                  })
                }else{
                  self.notifyError(self.ERROR['MANAGER_SERVICE']+3,self.name+": Function onURLDownloadComplete() failed : could not unzip URL package ");
                  self.log("Error downloading URL version.");
                  self.log("URL NOT unzipped !");
                  $("#starter").html("URL NOT unzipped !");
                  self.callback_updateURL("Error unzipFile");
                }
            });
        }else{
            self.notifyError(self.ERROR['MANAGER_SERVICE']+3,self.name+": Function onURLDownloadComplete() failed : could not Download URL package ");
            self.log("Error downloading URL version.");
            $("#starter").html("Error downloading URL version.");
            self.callback_updateURL("Error downloading URL version.");
        }
    };





    this.getLIC = function(callback){
      self.readFile("admira.lic", function(resultR, file){
        if(resultR){
          callback("admira.lic = "+resultR);
        }
        else{
          callback("error read admira.lic");
        }
      });
    }
    this.createLIC = function(data, callback){
      self.writeFile("admira.lic", data.lic, function(resultR, file){
        if(resultR){
          callback("admira.lic ("+data.lic+") created");
        }
        else{
          callback("admira.lic not created");
        }
      });
    }
    this.deleteAll = function(callback){
      self.readFile("admira.lic", function(resultR, file){
        if(resultR){
          if(resultR != null){
            var num = 0;
            self.deleteFilesInDir("documents", "", function(errD, resultD, number){
              num++;
              if(num >= number){
                self.writeFile("admira.lic", resultR, function(errW, resultW){
                  callback("deleteAll Completed -- Write admira.lic Completed -- Reboot");
                  self.rebootSystem();
                })
              }
            });
          }
        }
        else{
          callback("Error read admira.lic");
        }
      });
    }
    this.deleteAllAll = function(callback){
      var num = 0;
      self.deleteFilesInDir("documents", "", function(errD, resultD, number){
        num++;
        if(num >= number){
          callback("deleteAllAll Completed -- Reboot");
          self.rebootSystem();
        }
      });
    }
    this.clearCache = function(callback) {
      callback("no function");
    };
    this.calculateReboot = function(callback) {
      var today = self.getDate(self.server_time_activated);
      var tomorrow = self.getDate(self.server_time_activated);
      tomorrow.setDate(today.getDate()+1);
      tomorrow.setHours(4);
      tomorrow.setMinutes(0);
      var t1 = tomorrow.getTime();
      var t2 = today.getTime();
      var rest = t1-t2;
      if(rest >= 0){
          setTimeout(function(){
              self.notifyError(transfer.ERROR['TRANSFER_SERVICE']+17,self.name+": Proceed to perform the scheduled reboot");
              self.log("------------------------------------------------ proceed to perform the scheduled reboot");
              setTimeout(function(){self.rebootSystem();},3000);
          }, rest);
          self.log("------------------------------------------------programing reboot in "+rest+" milliseconds");
          if(typeof(callback) !== "undefined"){
            callback("programing reboot in "+rest+" milliseconds");
          }
      }
      else{
          self.log("------------------------------------------------no programing reboot");
          if(typeof(callback) !== "undefined"){
            callback("no programing reboot");
          }
      }
    };
    this.insertTimersOrReboot = function(reboot){
      self.getTimers(function(result){
        if(self.timersSystem.OnTimers[0][0] != 0 && self.timersSystem.OffTimers[0][0] != 0 ){
          if(self.timersSystem.OnTimers[0][0] != 403){
            self.calculateReboot();
          }
        }
        else{
          if(reboot == 1){
            self.insertTimerON(1,{hour:"04:03", week:":SUN:MON:TUE:WED:THU:FRI:SAT"}, function(err, result){
              self.insertTimerOFF(1,{hour:"04:00", week:":SUN:MON:TUE:WED:THU:FRI:SAT"}, function(err, result){
                self.log(" ---------------------- HAS POWER OFF / POWER ON PROGRAMED 4am - 4:03 am");
              })
            })
          }
          else{
            self.log(" ---------------------- HAS ¡¡NO!! POWER OFF / POWER ON PROGRAMED 4am - 4:03 am");
          }
        }
      })
    };
    this.copyUSB = function (callback){
      callback("no function");
    };
    this.deleteAll2 =  function(callback){
      callback("no function");
    };




    /* TIMERS */
    this.insertTimerON = function(timer, optionsInsertTimerON, callback_insertTimerON){
      self.log("optionsInsertTimerON-- TIMER:"+timer +"  "+optionsInsertTimerON.hour + "  "+optionsInsertTimerON.week);
      self.log(JSON.stringify(optionsInsertTimerON));
      b2bapis.b2bcontrol.setOnTimerRepeat("TIMER"+timer, optionsInsertTimerON.hour,"TIMER_MANUAL",optionsInsertTimerON.week, function() {
    	  self.log(".,.,.,.,.,.,..,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,");
      	self.log("-------------------------------------------addOnTimer successful -- ");
          callback_insertTimerON(null, true);
        },
        function(cbObject) {
      	  self.log("--------------------------------------------------------------------------------------------------------------------------------------------------------------------");
      	self.log(JSON.stringify(cbObject));
          callback_insertTimerON(cbObject);
        })
    };
    this.insertTimerOFF = function(timer, optionsInsertTimerOFF, callback_insertTimerOFF){
      function successCbOff() {
    	self.log("-------------------------------------------addOffTimer successful -- ");
        callback_insertTimerOFF(null, true);
      }
      function failureCbOff(cbObject) {
    	self.log(JSON.stringify(cbObject));
        callback_insertTimerOFF(cbObject);
      }
      self.log("optionsInsertTimerOFF -- timer:"+timer);
      self.log(JSON.stringify(optionsInsertTimerOFF));
      b2bapis.b2bcontrol.setOffTimerRepeat("TIMER"+timer,optionsInsertTimerOFF.hour,"TIMER_MANUAL",optionsInsertTimerOFF.week, successCbOff, failureCbOff);
    }


    this.timersOnNews = [];
    this.timersOnOld = [];
    this.timersOffNews = [];
    this.timersOffOld = [];
    this.calculateTimersOn = function(timersOn, callbackCalculateTimersOn){
      self.log("calculateTimersOn");
      self.timersOnNews = [];
      var timersOnHour = new Array(24);
      //var timersOn = null;
      for(var i=0; i<timersOn.length; i++){
        self.log("timersOn[i].day: "+timersOn[i].day);
        switch(timersOn[i].day) {
          case 0:
            if(timersOnHour[timersOn[i].hour] == undefined){
              timersOnHour[timersOn[i].hour] = "SUN";
            }
            else{
              timersOnHour[timersOn[i].hour] = timersOnHour[timersOn[i].hour] + ":SUN";
            }
            break;
          case 1:
            if(timersOnHour[timersOn[i].hour] == undefined){
              timersOnHour[timersOn[i].hour] = "MON";
            }
            else{
              timersOnHour[timersOn[i].hour] = timersOnHour[timersOn[i].hour] + ":MON";
            }
            break;
          case 2:
            if(timersOnHour[timersOn[i].hour] == undefined){
              timersOnHour[timersOn[i].hour] = "TUE";
            }
            else{
              timersOnHour[timersOn[i].hour] = timersOnHour[timersOn[i].hour] + ":TUE";
            }
            break;
          case 3:
            if(timersOnHour[timersOn[i].hour] == undefined){
              timersOnHour[timersOn[i].hour] = "WED";
            }
            else{
              timersOnHour[timersOn[i].hour] = timersOnHour[timersOn[i].hour] + ":WED";
            }
            break;
          case 4:
            if(timersOnHour[timersOn[i].hour] == undefined){
              timersOnHour[timersOn[i].hour] = "THU";
            }
            else{
              timersOnHour[timersOn[i].hour] = timersOnHour[timersOn[i].hour] + ":THU";
            }
            break;
          case 5:
            if(timersOnHour[timersOn[i].hour] == undefined){
              timersOnHour[timersOn[i].hour] = "FRI";
            }
            else{
              timersOnHour[timersOn[i].hour] = timersOnHour[timersOn[i].hour] + ":FRI";
            }
            break;
          case 6:
            if(timersOnHour[timersOn[i].hour] == undefined){
              timersOnHour[timersOn[i].hour] = "SAT";
            }
            else{
              timersOnHour[timersOn[i].hour] = timersOnHour[timersOn[i].hour] + ":SAT";
            }
            break;
        }
        if(i == timersOn.length-1){
          for(var j=0; j<24; j++){
            if(timersOnHour[j] != undefined && timersOnHour[j] != "undefined" && timersOnHour[j] != null){
              self.timersOnNews.push({
                "hour" : j+":00",
                "week" :  timersOnHour[j]
              });
            }
            if(j == 23){
              self.log(JSON.stringify(self.timersOnNews));
              callbackCalculateTimersOn();
            }
          }
        }
      }
    }
    this.calculateTimersOff = function(timersOff, callbackCalculateTimersOff){
      self.timersOffNews = [];
      var timersOffHour = new Array(24);
      for(var x=0; x<timersOff.length; x++){
        self.log(timersOff[x].day);
        switch(timersOff[x].day) {
          case 0:
            if(timersOffHour[timersOff[x].hour] == undefined){
              timersOffHour[timersOff[x].hour] = "SUN";
            }
            else{
              timersOffHour[timersOff[x].hour] = timersOffHour[timersOff[x].hour] + ":SUN";
            }
            break;
          case 1:
            if(timersOffHour[timersOff[x].hour] == undefined){
              timersOffHour[timersOff[x].hour] = "MON";
            }
            else{
              timersOffHour[timersOff[x].hour] = timersOffHour[timersOff[x].hour] + ":MON";
            }
            break;
          case 2:
            if(timersOffHour[timersOff[x].hour] == undefined){
              timersOffHour[timersOff[x].hour] = "TUE";
            }
            else{
              timersOffHour[timersOff[x].hour] = timersOffHour[timersOff[x].hour] + ":TUE";
            }
            break;
          case 3:
            if(timersOffHour[timersOff[x].hour] == undefined){
              timersOffHour[timersOff[x].hour] = "WED";
            }
            else{
              timersOffHour[timersOff[x].hour] = timersOffHour[timersOff[x].hour] + ":WED";
            }
            break;
          case 4:
            if(timersOffHour[timersOff[x].hour] == undefined){
              timersOffHour[timersOff[x].hour] = "THU";
            }
            else{
              timersOffHour[timersOff[x].hour] = timersOffHour[timersOff[x].hour] + ":THU";
            }
            break;
          case 5:
            if(timersOffHour[timersOff[x].hour] == undefined){
              timersOffHour[timersOff[x].hour] = "FRI";
            }
            else{
              timersOffHour[timersOff[x].hour] = timersOffHour[timersOff[x].hour] + ":FRI";
            }
            break;
          case 6:
            if(timersOffHour[timersOff[x].hour] == undefined){
              timersOffHour[timersOff[x].hour] = "SAT";
            }
            else{
              timersOffHour[timersOff[x].hour] = timersOffHour[timersOff[x].hour] + ":SAT";
            }
            break;
        }
        if(x == timersOff.length-1){
          for(var h=0; h<24; h++){
            if(timersOffHour[h] != undefined && timersOffHour[h] != "undefined"){
              self.timersOffNews.push({
                "hour" : h+":00",
                "week" :  timersOffHour[h]
              });
            }
            if(h == 23){
              self.log(JSON.stringify(self.timersOffNews));
              callbackCalculateTimersOff();
            }
          }
        }
      }
    }

    this.controlFlowTempOff = 0;
    this.controlFlowOff = function(){
      if(self.controlFlowTempOff < self.timersOffNews.length){
          self.insertTimerOFF(self.controlFlowTempOff+1, self.timersOffNews[self.controlFlowTempOff], function(err, result){});
          self.controlFlowTempOff++;
          self.controlFlowOff();
        }
        else{
            self.log("--------------------------- FINISH INSERT TIMERS -----------------------------");
        }
    }
    this.controlFlowTempOn = 0;
    this.controlFlowOn = function(){
		if(self.controlFlowTempOn < self.timersOnNews.length){
	        self.insertTimerON(self.controlFlowTempOn+1, self.timersOnNews[self.controlFlowTempOn], function(err, result){});
	        self.controlFlowTempOn++;
	        self.controlFlowOn();
	      }
	      else{
	          self.controlFlowTempOff = 0;
	          self.controlFlowOff();
	      }
    }

    this.calculateTimers = function(timersOn, timersOff){
        self.deteleTimers(function(){
          self.calculateTimersOn(timersOn, function(){
            self.calculateTimersOff(timersOff, function(){
              self.controlFlowTempOn = 0;
              self.controlFlowOn();
            })
          })
        })
    }

    this.deteleAllTimers = function(){
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER1", "00:00", "TIMER_OFF", "", function(){self.log("timer1 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER2", "00:00", "TIMER_OFF", "", function(){self.log("timer2 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER3", "00:00", "TIMER_OFF", "", function(){self.log("timer3 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER4", "00:00", "TIMER_OFF", "", function(){self.log("timer4 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER5", "00:00", "TIMER_OFF", "", function(){self.log("timer5 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER6", "00:00", "TIMER_OFF", "", function(){self.log("timer6 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER7", "00:00", "TIMER_OFF", "", function(){self.log("timer7 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});

	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER1", "00:00", "TIMER_OFF", "", function(){self.log("timer1 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER2", "00:00", "TIMER_OFF", "", function(){self.log("timer2 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER3", "00:00", "TIMER_OFF", "", function(){self.log("timer3 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER4", "00:00", "TIMER_OFF", "", function(){self.log("timer4 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER5", "00:00", "TIMER_OFF", "", function(){self.log("timer5 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER6", "00:00", "TIMER_OFF", "", function(){self.log("timer6 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER7", "00:00", "TIMER_OFF", "", function(){self.log("timer7 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
   }
    this.deteleTimers = function(callback){
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER1", "00:00", "TIMER_OFF", "", function(){self.log("timer1 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER2", "00:00", "TIMER_OFF", "", function(){self.log("timer2 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER3", "00:00", "TIMER_OFF", "", function(){self.log("timer3 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER4", "00:00", "TIMER_OFF", "", function(){self.log("timer4 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER5", "00:00", "TIMER_OFF", "", function(){self.log("timer5 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER6", "00:00", "TIMER_OFF", "", function(){self.log("timer6 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOffTimerRepeat("TIMER7", "00:00", "TIMER_OFF", "", function(){self.log("timer7 -- timer_off -- delete")}, function(e){self.log(JSON.stringify(e))});

	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER1", "00:00", "TIMER_OFF", "", function(){self.log("timer1 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER2", "00:00", "TIMER_OFF", "", function(){self.log("timer2 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER3", "00:00", "TIMER_OFF", "", function(){self.log("timer3 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER4", "00:00", "TIMER_OFF", "", function(){self.log("timer4 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER5", "00:00", "TIMER_OFF", "", function(){self.log("timer5 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER6", "00:00", "TIMER_OFF", "", function(){self.log("timer6 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   b2bapis.b2bcontrol.setOnTimerRepeat("TIMER7", "00:00", "TIMER_OFF", "", function(){self.log("timer7 -- timer_on -- delete")}, function(e){self.log(JSON.stringify(e))});
	   callback();
   }

    this.getTimers = function(callback){
      self.timersSystem = {};
    	var OnTimerRepeat = [[],[],[],[],[],[],[]];
  		try {
  			OnTimerRepeat[0][0] = b2bapis.b2bcontrol.getOnTimer("TIMER1");
  			OnTimerRepeat[0][1] = b2bapis.b2bcontrol.getOnTimerRepeat("TIMER1");
  		} catch (e) {
  			self.log("[OnTimerRepeat] call syncFunction exception [" + e.code + "[ name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OnTimerRepeat[1][0] = b2bapis.b2bcontrol.getOnTimer("TIMER2");
  			OnTimerRepeat[1][1] = b2bapis.b2bcontrol.getOnTimerRepeat("TIMER2");
  		} catch (e) {
  			self.log("[OnTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OnTimerRepeat[2][0] = b2bapis.b2bcontrol.getOnTimer("TIMER3");
  			OnTimerRepeat[2][1] = b2bapis.b2bcontrol.getOnTimerRepeat("TIMER3");
  		} catch (e) {
  			self.log("[OnTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OnTimerRepeat[3][0] = b2bapis.b2bcontrol.getOnTimer("TIMER4");
  			OnTimerRepeat[3][1] = b2bapis.b2bcontrol.getOnTimerRepeat("TIMER4");
  		} catch (e) {
  			self.log("[OnTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OnTimerRepeat[4][0] = b2bapis.b2bcontrol.getOnTimer("TIMER5");
  			OnTimerRepeat[4][1] = b2bapis.b2bcontrol.getOnTimerRepeat("TIMER5");
  		} catch (e) {
  			self.log("[OnTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OnTimerRepeat[5][0] = b2bapis.b2bcontrol.getOnTimer("TIMER6");
  			OnTimerRepeat[5][1] = b2bapis.b2bcontrol.getOnTimerRepeat("TIMER6");
  		} catch (e) {
  			self.log("[OnTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OnTimerRepeat[6][0] = b2bapis.b2bcontrol.getOnTimer("TIMER7");
  			OnTimerRepeat[6][1] = b2bapis.b2bcontrol.getOnTimerRepeat("TIMER7");
  		} catch (e) {
  			self.log("[OnTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		var OffTimerRepeat =  [[],[],[],[],[],[],[]];
  		try {
  			OffTimerRepeat[0][0] = b2bapis.b2bcontrol.getOffTimer("TIMER1");
  			OffTimerRepeat[0][1] = b2bapis.b2bcontrol.getOffTimerRepeat("TIMER1");
  		} catch (e) {
  			self.log("[getOffTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OffTimerRepeat[1][0] = b2bapis.b2bcontrol.getOffTimer("TIMER2");
  			OffTimerRepeat[1][1] = b2bapis.b2bcontrol.getOffTimerRepeat("TIMER2");
  		} catch (e) {
  			self.log("[getOffTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OffTimerRepeat[2][0] = b2bapis.b2bcontrol.getOffTimer("TIMER3");
  			OffTimerRepeat[2][1] = b2bapis.b2bcontrol.getOffTimerRepeat("TIMER3");
  		} catch (e) {
  			self.log("[getOffTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OffTimerRepeat[3][0] = b2bapis.b2bcontrol.getOffTimer("TIMER4");
  			OffTimerRepeat[3][1] = b2bapis.b2bcontrol.getOffTimerRepeat("TIMER4");
  		} catch (e) {
  			self.log("[getOffTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OffTimerRepeat[4][0] = b2bapis.b2bcontrol.getOffTimer("TIMER4");
  			OffTimerRepeat[4][1] = b2bapis.b2bcontrol.getOffTimerRepeat("TIMER5");
  		} catch (e) {
  			self.log("[getOffTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OffTimerRepeat[5][0] = b2bapis.b2bcontrol.getOffTimer("TIMER6");
  			OffTimerRepeat[5][1] = b2bapis.b2bcontrol.getOffTimerRepeat("TIMER6");
  		} catch (e) {
  			self.log("[getOffTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
  		try {
  			OffTimerRepeat[6][0] = b2bapis.b2bcontrol.getOffTimer("TIMER7");
  			OffTimerRepeat[6][1] = b2bapis.b2bcontrol.getOffTimerRepeat("TIMER7");
  		} catch (e) {
  			self.log("[getOffTimerRepeat] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
  		}
		  self.log(JSON.stringify(OnTimerRepeat));
		  self.log(JSON.stringify(OffTimerRepeat));
      self.timersSystem.OnTimers = OnTimerRepeat;
      self.timersSystem.OffTimers = OffTimerRepeat;
		  callback(self.timersSystem);
    }

    this.upgradeApplication = function (callback_upgradeApplication){
      self.updateURL(function(err_updateURL,result_updateURL){
        if(!err_updateURL){
          callback_upgradeApplication(result_updateURL);
        }
        else{
          callback_upgradeApplication(err_updateURL);
          self.rebootSystem();
        }
      });
    };

    this.upgradeApplicationInsert = function(url, callback_upgradeApplicationInsert){
        var onSuccess = function(val) {
            //callback_upgradeApplicationInsert(val);
            var origin = url + "/ADmira.wgt";
            self.downloadFile(origin,"",self.onURLDownloadComplete,function(err, result){
              if(!err){
                callback_upgradeApplicationInsert("Update true : "+origin);
                self.rebootSystem();
              }
              else{
                callback_upgradeApplicationInsert(JSON.stringify(err));
              }
            });
        };
        var onError = function(error) {
            //console.log("[setURLLauncherAddress] code :" + error.code + " error name: " + error.name + "  message " + error.message);
            callback_upgradeApplicationInsert(JSON.stringify(error));
        };
        console.log("[setURLLauncherAddress] ");
        b2bapis.b2bcontrol.setURLLauncherAddress(url, onSuccess, onError);
    }
    this.upgradeApplicationInstall = function(callback_upgradeApplicationInstall){
      var onSuccess = function(val) {
          var origin = "http://install.admira.com/tizen/ADmira.wgt";
          self.downloadFile(origin,"",self.onURLDownloadComplete,function(err, result){
            if(!err){
              callback_upgradeApplicationInstall("Update true : "+origin);
              self.rebootSystem();
            }
            else{
              callback_upgradeApplicationInstall(JSON.stringify(err));
            }
          });
      };
      var onError = function(error) {
          callback_upgradeApplicationInstall(JSON.stringify(error));
      };
      console.log("[setURLLauncherAddress] ");
      b2bapis.b2bcontrol.setURLLauncherAddress("http://install.admira.com/tizen", onSuccess, onError);
    }
    this.upgradeApplicationNow = function(callback_upgradeApplicationNow){
      self.updateURL(function(err_updateURL,result_updateURL){
        if(!err_updateURL){
          callback_upgradeApplicationNow(result_updateURL);
        }
        else{
          callback_upgradeApplicationNow(err_updateURL);
          self.rebootSystem();
        }
      });
    }
    this.updateApplicationS = function(callback_updateApplicationS){
        callback_updateApplicationS("no function");
    }

    this.generateNewScreenController = function ( screen ){
    	var controller = false;
		return controller;
	}

    this.getStatusPanel = function(callback_getStatusPanel){
        try{
            var PanelMuteStatus = b2bapis.b2bcontrol.getPanelMuteStatus();
            self.log("..................................PanelMuteStatus");
            self.log(PanelMuteStatus);
            if(PanelMuteStatus == "OFF"){
                callback_getStatusPanel(null, true);
            }
            else{
                callback_getStatusPanel(null, false);
            }
        }
        catch(e){
            callback_getStatusPanel(JSON.stringify(e));
        }
    }
    this.setPanelOff = function(callback_setPanelOff){
        var onSuccess = function(val) {
            self.log("..................................off val");
            self.log(val);
            if(typeof(callback_setPanelOff) != "undefined"){
              callback_setPanelOff(null, val);
            }
        };
        var onError = function(error) {
            self.log("[setPanelMute] code :" + error.code + " error name: " + error.name + "  message " + error.message);
            callback_setPanelOff(JSON.stringify(error));
        };
        console.log("[setPanelMute] ");
        b2bapis.b2bcontrol.setPanelMute("ON", onSuccess, onError);
    }
    this.setPanelOn = function(callback_setPanelOn){
        var onSuccess = function(val) {
            self.log("..................................on val");
            self.log(val);
            if(typeof(callback_setPanelOn) != "undefined"){
              callback_setPanelOn(null, val);
            }
        };
        var onError = function(error) {
            self.log("[setPanelMute] code :" + error.code + " error name: " + error.name + "  message " + error.message);
            if(typeof(callback_setPanelOn) != "undefined"){
              callback_setPanelOn(JSON.stringify(error));
            }
        };
        console.log("[setPanelMute] ");
        b2bapis.b2bcontrol.setPanelMute("OFF", onSuccess, onError);
    }
    this.updateFirmware = function(data, callback_updateFirmware){
      //callback_updateFirmware("no function");
      var onSuccess = function(val) {
        self.log("[updateFirmware] success : " + val);
       };
       var onError = function(error) {
        self.log("[updateFirmware] code :" + error.code + " error name: " + error.name + "  message " + error.message);
       };
       self.log("[updateFirmware] ");
       var SoftwareID   = data.ID;
       var SWUFileName  = data.fileName;
       var SWVersion  = data.version;
       var SWURL    = data.url;
       var SWTotalBytes  = data.totalBytes;
       b2bapis.b2bcontrol.updateFirmware(SoftwareID, SWUFileName, SWVersion, SWURL, SWTotalBytes, onSuccess, onError);
    };
    this.changePictureMode = function(callback_changePictureMode){
        callback_changePictureMode("no function");
    }
    this.getPictureProperty = function(callback_getPictureProperty){
        callback_getPictureProperty("no function");
    }
    this.setPortraitMode = function(data, callback_setPortraitMode){
        callback_setPortraitMode("no function");
    }
    this.getContentRotation = function(callback_getContentRotation){
        callback_getContentRotation("no function");
    }
    this.setContentRotation = function(data, callback_setContentRotation){
        callback_setContentRotation("no function");
    }
    this.getVideoStatus = function(callback_getVideoStatus){
        callback_getVideoStatus("no function");
    }
    this.checkSize = function(filename,callback,params){
        callback(false)
    };
    this.shellExec = function(command,callback,params){
      var result = "";
      if(typeof(callback) !== "undefined"){
          callback(result,command,params);
      }
    };

    this.socketServerBroadcast = function(msg){
      if(self.websocket_server){
        // TODO
        self.log("[NodeJS] : Trying to broadcast !");
        try{
          if(self.nodejs_service){
            self.nodejs_service.emit("broadcast",msg);
            self.log("[NodeJS] : Broadcast sent !");
          }
        }catch(e){
          self.log("Error creating Socket.IO Connector for WebSocket Server");
        }
      }
    };

    this.socketServerStart = function(port,onOpenCallback,onMessageCallback,onTerminatedCallback){
      if(self.nodejs_service){
        if(!self.websocket_server){
          self.websocket_server = true;
          if(typeof(onOpenCallback) !== "undefined"){
            self.websocket_onOpenCallback = onOpenCallback;
          }
          if(typeof(onMessageCallback) !== "undefined"){
           self.websocket_onMessageCallback = onMessageCallback;
          }
          if(typeof(onTerminatedCallback) !== "undefined"){
           self.websocket_onTerminatedCallback = onTerminatedCallback;
          }

          if(self.system_ip != "0.0.0.0"){
            self.nodejs_service.on('onBroadcastComplete', function(data){
              self.log("[NodeJS] - onBroadcastComplete : "+JSON.stringify(data));
            });
            self.nodejs_service.on('onSocketMessage', function(data){
              self.log("[NodeJS] - onSocketMessage : "+JSON.stringify(data));
              self.websocket_onMessageCallback(data);
            });
            self.nodejs_service.on('onSocketOpen', function(data){
              self.log("[NodeJS] - onSocketOpen : "+JSON.stringify(data));
            });
            self.nodejs_service.on('onSocketClose', function(data){
              self.log("[NodeJS] - onSocketClose : "+JSON.stringify(data));
            });
            self.nodejs_service.on('onSocketError', function(data){
              self.log("[NodeJS] - onSocketError : "+JSON.stringify(data));
            });
            self.nodejs_service.on('onInitWebSocketServer', function(data){
              self.log("[NodeJS] - onInitWebSocketServer : "+JSON.stringify(data));
            });
            self.nodejs_service.emit("initWebSocketServer",port);
          }
        }
      }else{
        self.startServiceNodeJS();
      }
    };

    this.findSyncIp = function (callback){
        try{
          var interfaces = self.getNetworkInfo();
          if(interfaces.length > 0){
            if(typeof(interfaces[0].ip) !== "undefined"){
              if(typeof(callback) !== "undefined"){
                callback(interfaces[0].ip);
              }
            }else{
              if(typeof(callback) !== "undefined"){
                callback("0.0.0.0");
              }
            }
          }else{
            if(typeof(callback) !== "undefined"){
              callback("0.0.0.0");
            }
          }
        }catch(e){
          if(typeof(callback) !== "undefined"){
            callback("0.0.0.0");
          }
        }
    };

    this.getPowerOnOffHistory = function(callback_getPowerOnOffHistory){
        callback_getPowerOnOffHistory("no function");
    }

    this.getBrigthness = function(callback_getBrigthness){
        //var res_brigthness = webapis.avinfo.getBrightness();
        callback_getBrigthness(webapis.avinfo.getBrightness());
    }
    this.setBrightness = function(brightness, callback_setBrightness){
        var result = webapis.avinfo.setBrightness(parseInt(brightness));
        callback_setBrightness(result);
    }
    this.getContrast = function(callback_getContrast){
        var res_contrast = webapis.avinfo.getContrast();
        callback_getContrast(res_contrast);
    }
    this.setContrast = function(contrast, callback_setContrast){
        var result = webapis.avinfo.setContrast(parseInt(contrast))
        callback_setContrast(result);
    }
    this.getBacklight = function(callback_getBacklight){
        var res_backlight = webapis.avinfo.getBacklight();
        callback_getBacklight(res_backlight);
    }
    this.setBacklight = function(backlight, callback_setBacklight){
        var result = webapis.avinfo.setBacklight(parseInt(backlight));
        callback_setBacklight(result);
    }
    this.geConfigPanel = function(callback_geConfigPanel){
        var res_brigthness = webapis.avinfo.getBrightness();
        var res_contrast = webapis.avinfo.getContrast();
        var res_backlight = webapis.avinfo.getBacklight();
        var total = {
          brigthness : res_brigthness,
          contrast : res_contrast,
          backlight : res_backlight
        }
        callback_geConfigPanel(total);
    }
    this.setConfigPanel = function(data, callback_setConfigPanel){
        var resultbrightness = webapis.avinfo.setBrightness(parseInt(data.brightness));
        var resultcontrast = webapis.avinfo.setContrast(parseInt(data.contrast))
        var resultbacklight = webapis.avinfo.setBacklight(parseInt(data.backlight));
        var resultTotal = {
          brigthness : resultbrightness,
          contrast : resultcontrast,
          backlight : resultbacklight
        }
        callback_setConfigPanel(resultTotal);
    }
    this.setNativePortraitMode = function(data, mode, callback_setNativePortraitMode){
        callback_setNativePortraitMode("no function");
    }
    this.getStatusRemoteControl = function(callback_getStatusRemoteControl){
      var IRLockOnOff = null;
       try {
        IRLockOnOff = b2bapis.b2bcontrol.getIRLockStatus();
       } catch (e) {
         callback_getStatusRemoteControl(e)
        console.log("[getIRLockOnOff] call syncFunction exception [" + e.code + "] name: " + e.name + " message: " + e.message);
       }
      callback_getStatusRemoteControl(IRLockOnOff)
    }
    this.setStatusRemoteControl = function(data, callback_setStatusRemoteControl){
        var onSuccess = function(val) {
          callback_setStatusRemoteControl(val);
        };
        var onError = function(error) {
          callback_setStatusRemoteControl(error);
        };
        if(data == "OFF"){
          b2bapis.b2bcontrol.setIRLockOnOff("OFF", onSuccess, onError);
        }
        else{
          b2bapis.b2bcontrol.setIRLockOnOff("ON", onSuccess, onError);
        }
    }
    
    this.socketSetVolume = function(new_volume){
      self.setVolume(new_volume);
    };
    
    /// GET OPERATING SYSTEM INFO
    this.getInfo = function(include_system_info){
      if(typeof(include_system_info) === "undefined"){include_system_info = false;}
      var info = {};	
      try{
        if(include_system_info){
          if(self.dataInfoSystem){
            try{
              self.dataInfoSystem.launcher_url = b2bapis.b2bcontrol.getURLLauncherAddress();
            }catch(e){
              self.log("Could not getURLLauncherAddress()");
            }
          }
          info['system'] = {'type':'sssp','versions':self.sssp_version,'details':self.dataInfoSystem};
        }
        info['cpu'] = {'uptime':b2bapis.b2bcontrol.getSystemUpTime(),'load':self.cpu_usage};
        info['mem'] = {'total':tizen.systeminfo.getTotalMemory(),'free':tizen.systeminfo.getAvailableMemory()};
        if(self.storage){
          info['storage'] = [];
          for(var i=0;i<self.storage.units.length;i++){
            info['storage'].push({'type':self.storage.units[i].type,'rootDir':self.rootDir,'appDir':self.appDir,'total':self.storage.units[i].capacity,'free':self.storage.units[i].availableCapacity});
          }
        }
        info['ifaces'] = self.getNetworkInfo();
        info['displays'] = self.display_screens;
        self.getInfoSystemPlus();
      }catch(e){
        self.log("Could not getInfo() from OS object");
      }
      return info;
    };
}
TizenSystemController.prototype = new SystemController();
